/*
Created by: X
http://www.createdbyx.com/

The zeeAni library is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using zeeAni.Properties;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using System.Threading;

namespace zeeAni
{
    /// <summary>
    /// Delegate used to convert a string value into a accessor key value.
    /// </summary>
    /// <typeparam name="Key">Specifies the type of the key to be used.</typeparam>
    /// <param name="value">The string value to be converted.</param>
    /// <param name="k">Returns the key that was converted from the value if succssfull.</param>
    /// <returns>Returns true if the conversion was successfull.</returns>
    public delegate bool KeyConverter<Key>(string value, out Key k);

    /// <summary>
    /// Used to load animation data from a file or game asset.
    /// </summary>
    /// <typeparam name="Key">The type used as the accessor key.</typeparam>
    /// <typeparam name="FrameType">The type used to represent a frame of animation.</typeparam>
    public class AnimationLoader<Key, FrameType> : IDisposable where FrameType : SpriteFrame, new()
    {

        /// <summary>
        /// Holds a reference to a content manager.
        /// </summary>
        private ContentManager contentManager;
        /// <summary>
        /// Holds a reference to a AnimationSet object used to store the animation data.
        /// </summary>
        private AnimationSet<Key, FrameType> animation;
        /// <summary>
        /// Holds an error message is an error occoured.
        /// </summary>
        private string errorMsg = string.Empty;
        /// <summary>
        /// Holds a reference to a key converter used to convert strings into an acsessor key.
        /// </summary>
        private KeyConverter<Key> keyConvert;


        /// <summary>
        /// Gets/Sets a reference to a key converter used to convert strings into an acsessor key.
        /// </summary>
        public KeyConverter<Key> Converter
        {
            get { return keyConvert; }
            set { keyConvert = value; }
        }

        /// <summary>
        /// Gets the last error message that occoured.
        /// </summary>
        public string ErrorMessage
        {
            get { return errorMsg; }
        }

        /// <summary>
        /// Gets a reference to the animatin data that was loaded.
        /// </summary>
        /// <remarks>This is replaced with new animation data every time a call to Load succsessfully completes.</remarks>
        /// <typeparam name="Key">The type used as the accessor key.</typeparam>
        /// <typeparam name="FrameType">The type used to represent a frame of animation.</typeparam>
        public AnimationSet<Key, FrameType> Animation
        {
            get { return this.animation; }
        }

        /// <summary>
        /// Initilizes a new instance of the AnimationLoader class.
        /// </summary>
        /// <param name="contentManager">A reference to a ContentManager class that will be used to load the animation data.</param>
        public AnimationLoader(ContentManager contentManager)
        {
            this.contentManager = contentManager;
        }

        /// <summary>
        /// Initilizes a new instance of the AnimationLoader class.
        /// </summary>
        /// <param name="contentManager">A reference to a ContentManager class that will be used to load the animation data.</param>
        /// <param name="kc">A reference to a key converter delegate that will be used when a key conversion is required.</param>
        public AnimationLoader(ContentManager contentManager, KeyConverter<Key> kc)
        {
            this.contentManager = contentManager;
            this.keyConvert = kc;
        }

        /// <summary>
        /// Loads a animation from a xml string.
        /// </summary>
        /// <param name="contentManager">A reference to a ContentManager class that will be used to load the animation data.</param>
        /// <param name="xml">A string that holds the xml animation data.</param>
        /// <param name="animation">Will return a a AnimationSet object if load was succsessfull.</param>
        /// <param name="kc">A reference to a key converter delegate that will be used when a key conversion is required.</param>
        /// <param name="err">A string containing a error message if an error occoured.</param>
        /// <returns>Returns true if the animation was loaded successfully.</returns>
        public static bool LoadAnimation(ContentManager contentManager, string xml, out  AnimationSet<Key, FrameType> animation, KeyConverter<Key> kc, out string err)
        {
            AnimationLoader<Key, FrameType> loader = new AnimationLoader<Key, FrameType>(contentManager);
            loader.keyConvert = kc;
            bool ret = loader.LoadXml(xml);
            animation = loader.animation;
            err = loader.errorMsg;
            loader.Dispose();
            loader = null;
            return ret;
        }

        /// <summary>
        /// Loads a animation from a XMLDocument object.
        /// </summary>
        /// <param name="contentManager">A reference to a ContentManager class that will be used to load the animation data.</param>
        /// <param name="doc">A reference to a XMLDocument object that holds the xml animation data.</param>
        /// <param name="animation">Will return a a AnimationSet object if load was succsessfull.</param>
        /// <param name="kc">A reference to a key converter delegate that will be used when a key conversion is required.</param>
        /// <param name="err">A string containing a error message if an error occoured.</param>
        /// <returns>Returns true if the animation was loaded successfully.</returns>
        public static bool LoadAnimation(ContentManager contentManager, XmlDocument doc, out  AnimationSet<Key, FrameType> animation, KeyConverter<Key> kc, out string err)
        {
            AnimationLoader<Key, FrameType> loader = new AnimationLoader<Key, FrameType>(contentManager);
            loader.keyConvert = kc;
            bool ret = loader.Load(doc);
            animation = loader.animation;
            err = loader.errorMsg;
            loader.Dispose();
            loader = null;
            return ret;
        }

        /// <summary>
        /// Loads a animation from a XMLDocument object.
        /// </summary>
        /// <param name="doc">A reference to a XMLDocument object that holds the xml animation data.</param>
        /// <returns>Returns true if the animation was loaded successfully.</returns>
        public bool Load(XmlDocument doc)
        {
            if (doc == null) throw new ArgumentNullException("doc");

            // check for proper root element
            if (string.Compare(doc.DocumentElement.Name, Resources.XML_animation) != 0)
            {
                this.errorMsg = Resources.CouldNotLoadStream;
                return false;
            }

            // System.Xml.XmlNodeList animationItems = doc.DocumentElement.GetElementsByTagName(Resources.XML_animation);
            return this.DoLoad(doc.DocumentElement);
        }

        /// <summary>
        /// Loads a animation from a xml string.
        /// </summary>
        /// <param name="xml">A string that holds the xml animation data.</param>
        /// <returns>Returns true if the animation was loaded successfully.</returns>
        public bool LoadXml(string xml)
        {
            if (string.IsNullOrEmpty(xml)) throw new ArgumentNullException("xml");

            XmlDocument doc = new XmlDocument();
            doc.LoadXml(xml);
            return this.Load(doc);
        }

        /// <summary>
        /// Parses animation information from a xml node.
        /// </summary>
        /// <param name="aniInfos">A reference to a generic dictionary that will hold the animation info data.</param>
        /// <param name="aa">The animation node that will be parsed.</param>
        /// <returns>Returns true if the data was loaded successfully.</returns>
        protected internal bool ParseAnimationInfo(Dictionary<Key, AnimationInfo> aniInfos, XmlNode aa)
        {
            Dictionary<string, string> att;
            Helpers.ParseAttributes(aa, out att);
            // check for name element
            if (att.ContainsKey(Resources.XML_name) && att.ContainsKey(Resources.XML_startframe) && att.ContainsKey(Resources.XML_endframe) &&
                (att.ContainsKey(Resources.XML_loops) || att.ContainsKey(Resources.XML_looping)) && (att.ContainsKey(Resources.XML_framerate) || att.ContainsKey(Resources.XML_fps)))
            {
                Key tmpname = default(Key);
                int sfme;
                int efrme;
                bool l = false;
                int fps = 1;

                // try to convert the key
                if (!this.keyConvert(att[Resources.XML_name].Trim(), out tmpname))
                {
                    this.errorMsg = Resources.ERR_KeyConverterCouldNotConvertKey;
                    return false;
                }

                if (!int.TryParse(att[Resources.XML_startframe], out sfme))
                {
                    this.errorMsg = Resources.ExpectedNamedType.Replace(Resources.RKEY, Resources._integer);
                    return false;
                }
                if (!int.TryParse(att[Resources.XML_endframe], out efrme))
                {
                    this.errorMsg = Resources.ExpectedNamedType.Replace(Resources.RKEY, Resources._integer);
                    return false;
                }
                if (att.ContainsKey(Resources.XML_loops) && !bool.TryParse(att[Resources.XML_loops], out l))
                {
                    this.errorMsg = Resources.ExpectedBool;
                    return false;
                }
                if (att.ContainsKey(Resources.XML_looping) && !bool.TryParse(att[Resources.XML_looping], out l))
                {
                    this.errorMsg = Resources.ExpectedBool;
                    return false;
                }
                if (att.ContainsKey(Resources.XML_framerate))
                {
                    if (!int.TryParse(att[Resources.XML_framerate], out fps))
                    {
                        this.errorMsg = Resources.ExpectedNamedType.Replace(Resources.RKEY, Resources._integer);
                        return false;
                    }
                }
                if (att.ContainsKey(Resources.XML_fps))
                {
                    if (!int.TryParse(att[Resources.XML_fps], out fps))
                    {
                        this.errorMsg = Resources.ExpectedNamedType.Replace(Resources.RKEY, Resources._integer);
                        return false;
                    }
                }

                // add info
                try
                {
                    aniInfos.Add(tmpname, new AnimationInfo(sfme, efrme, fps, l));
                }
                catch (Exception ex)
                {
                    this.errorMsg = ex.Message;
                    return false;
                }
            }
            else
            {
                XmlNode nodeName = aa.SelectSingleNode(Resources.XML_name);
                XmlNode nodeStart = aa.SelectSingleNode(Resources.XML_startframe);
                XmlNode nodeEnd = aa.SelectSingleNode(Resources.XML_endframe);
                XmlNode nodeFPS = aa.SelectSingleNode(Resources.XML_fps);
                if (nodeFPS == null) nodeFPS = aa.SelectSingleNode(Resources.XML_framerate);
                XmlNode nodeLoops = aa.SelectSingleNode(Resources.XML_loops);
                if (nodeLoops == null) nodeLoops = aa.SelectSingleNode(Resources.XML_looping);

                if (nodeName != null && nodeName.InnerText != string.Empty && nodeStart != null && nodeStart.InnerText != string.Empty &&
                    nodeEnd != null && nodeEnd.InnerText != string.Empty && nodeFPS != null && nodeFPS.InnerText != string.Empty &&
                    nodeLoops != null && nodeLoops.InnerText != string.Empty)
                {
                    int frameStart = -1;
                    int frameEnd = -1;
                    int frameFPS = -1;
                    bool looping = false;
                    if (!int.TryParse(nodeStart.InnerText, out frameStart))
                    {
                        this.errorMsg = Resources.ExpectedNamedType.Replace(Resources.RKEY, Resources._integer);
                        return false;
                    }
                    if (!int.TryParse(nodeEnd.InnerText, out frameEnd))
                    {
                        this.errorMsg = Resources.ExpectedNamedType.Replace(Resources.RKEY, Resources._integer);
                        return false;
                    }
                    if (!int.TryParse(nodeFPS.InnerText, out frameFPS))
                    {
                        this.errorMsg = Resources.ExpectedNamedType.Replace(Resources.RKEY, Resources._integer);
                        return false;
                    }
                    if (!bool.TryParse(nodeLoops.InnerText, out looping))
                    {
                        this.errorMsg = Resources.ExpectedBool;
                        return false;
                    }

                    // try to convert the key
                    Key tmpname = default(Key);
                    if (!this.keyConvert(nodeName.InnerText.Trim(), out tmpname))
                    {
                        this.errorMsg = Resources.ERR_KeyConverterCouldNotConvertKey;
                        return false;
                    }

                    try
                    {
                        // add the animation info
                        aniInfos.Add(tmpname, new AnimationInfo(frameStart, frameEnd, frameFPS, looping));
                    }
                    catch (Exception ex)
                    {
                        this.errorMsg = ex.Message;
                        return false;
                    }
                }
                else
                {
                    this.errorMsg = Resources.AddAnimationElementIsMissingsElements;
                    return false;
                }
            }

            return true;
        }

        /// <summary>
        /// Primary load method.
        /// </summary>
        /// <param name="animationNode">A reference to a xml node that contains the animation data.</param>
        /// <returns>Returns true if the data was loaded successfully.</returns>
        private bool DoLoad(XmlNode animationNode)
        {
            // try to find animation node and if it is not there try to read source rect nodes.
            // XmlNode animationNode = doc.DocumentElement.SelectSingleNode(Resources.XML_animation);
            List<FrameType> sf = new List<FrameType>();
            Dictionary<Key, AnimationInfo> aniInfos = new Dictionary<Key, AnimationInfo>();
            if (animationNode != null)
            {
                // add animations
                XmlNodeList anis = animationNode.SelectNodes(Resources.XML_addanimation);
                foreach (XmlNode aa in anis)
                {
                    if (!this.ParseAnimationInfo(aniInfos, aa)) return false;
                }

                // parse frame data
                XmlNodeList frames = animationNode.SelectNodes(Resources.XML_frame);
                foreach (XmlNode f in frames)
                {
                    Dictionary<string, string> att;
                    Helpers.ParseAttributes(f, out att);
                    int index;
                    index = int.MaxValue;
                    if (att.ContainsKey(Resources.XML_index))
                    {
                        int.TryParse(att[Resources.XML_index], out index);
                    }
                    else
                    {
                        // try reading index
                        if (!Helpers.ParseIntNode(f, Resources.XML_index, ref index, out this.errorMsg))
                        {
                          //  return false;
                        }
                    }

                    Rectangle sr = Rectangle.Empty;

                    // try to read srcrect element
                    if (Helpers.ParseNodeAttributes(f, Resources.XML_sourcerectangle, out att))
                    {

                        if (att.ContainsKey(Resources.XML_x)) int.TryParse(att[Resources.XML_x], out sr.X);
                        if (att.ContainsKey(Resources.XML_y)) int.TryParse(att[Resources.XML_y], out sr.Y);
                        if (att.ContainsKey(Resources.XML_left)) int.TryParse(att[Resources.XML_left], out sr.X);
                        if (att.ContainsKey(Resources.XML_top)) int.TryParse(att[Resources.XML_top], out sr.Y);
                        if (att.ContainsKey(Resources.XML_width)) int.TryParse(att[Resources.XML_width], out sr.Width);
                        if (att.ContainsKey(Resources.XML_height)) int.TryParse(att[Resources.XML_height], out sr.Height);
                    }
                    else
                    {
                        // try reading source rect info if no source rect is provided the full size of the image will be used
                        if (!Helpers.ParseIntNode(f, Resources.XML_sourceleft, ref sr.X, out this.errorMsg))
                        {
                           // return false;
                        }
                        if (!Helpers.ParseIntNode(f, Resources.XML_sourcetop, ref sr.Y, out this.errorMsg))
                        {
                           // return false;
                        }
                        if (!Helpers.ParseIntNode(f, Resources.XML_sourcewidth, ref sr.Width, out this.errorMsg))
                        {
                           // return false;
                        }
                        if (!Helpers.ParseIntNode(f, Resources.XML_sourceheight, ref sr.Height, out this.errorMsg))
                        {
                           // return false;
                        }
                    }

                    // try to read origin element
                    Vector2 ori = Vector2.Zero;
                    if (Helpers.ParseNodeAttributes(f, Resources.XML_origin, out att))
                    {

                        if (att.ContainsKey(Resources.XML_x)) float.TryParse(att[Resources.XML_x], out ori.X);
                        if (att.ContainsKey(Resources.XML_y)) float.TryParse(att[Resources.XML_y], out ori.Y);
                    }
                    else
                    {
                        // try reading origin info
                        if (!Helpers.ParseFloatNode(f, Resources.XML_x, ref ori.X, out this.errorMsg))
                        {
                            // return false;
                        }
                        if (!Helpers.ParseFloatNode(f, Resources.XML_y, ref ori.Y, out this.errorMsg))
                        {
                            // return false;
                        }
                    }

                    // try to read scale element
                    Vector2 sca = Vector2.One;
                    if (Helpers.ParseNodeAttributes(f, Resources.XML_scale, out att))
                    {

                        if (att.ContainsKey(Resources.XML_x)) float.TryParse(att[Resources.XML_x], out sca.X);
                        if (att.ContainsKey(Resources.XML_y)) float.TryParse(att[Resources.XML_y], out sca.Y);
                    }
                    else
                    {
                        // try reading origin info
                        if (!Helpers.ParseFloatNode(f, Resources.XML_x, ref sca.X, out this.errorMsg))
                        {
                            // return false;
                        }
                        if (!Helpers.ParseFloatNode(f, Resources.XML_y, ref sca.Y, out this.errorMsg))
                        {
                            // return false;
                        }
                    }

                    // check for texture asset element
                    string textureAssetName;
                    if (!Helpers.ParseStringNode(f, Resources.XML_texture, out textureAssetName, out this.errorMsg))
                    {
                        return false;
                    }

                    // look for a effect node
                    // check for texture asset element
                    string effect;
                    SpriteEffects fx = SpriteEffects.None;
                    if (Helpers.ParseStringNode(f, Resources.XML_effect, out effect, out this.errorMsg))
                    {
                        int tmpVal = (int)fx;
                        if (int.TryParse(effect, out tmpVal))
                        {
                            fx = (SpriteEffects)tmpVal;
                        }
                        else
                        {
                            try
                            {
                                fx = (SpriteEffects)Enum.Parse(typeof(SpriteEffects), effect, true);
                            }
                            catch (Exception ex)
                            {
                                this.errorMsg = ex.Message;
                                return false;
                            }
                        }
                    }

                    // create the frames
                    FrameType tmpframe = new FrameType();
                    tmpframe.SourceRectangle = sr;
                    tmpframe.origin = ori;
                    tmpframe.scale = sca;
                    tmpframe.effect = fx;
                   // tmpframe.textureAsset = textureAssetName;
                   // tmpframe.needsUpdate = true;

                    try
                    {
                        if (this.contentManager != null && !string.IsNullOrEmpty(textureAssetName))
                        {
                            tmpframe.texture = this.contentManager.Load<Texture2D>(textureAssetName);
                            if (tmpframe.sourceRect == Rectangle.Empty)
                            {
                                tmpframe.sourceRect = new Rectangle(0, 0, tmpframe.texture.Width, tmpframe.texture.Height);
                            }
                            //   tmpframe.needsUpdate = false;
                        }
                    }
                    catch (Exception ex)
                    {
                        this.errorMsg = ex.Message;
                        return false;
                    }

                    if (f.SelectSingleNode(Resources.XML_color) != null)
                    {
                        if (!Helpers.ParseColorNode(f, Resources.XML_color, out tmpframe.color, out this.errorMsg)) return false;
                    }
                    if (Helpers.ParseFloatNode(f, Resources.XML_rotation, ref tmpframe.rotation, out this.errorMsg))
                    {
                        tmpframe.rotation = MathHelper.ToRadians(tmpframe.rotation);
                    }
                    this.errorMsg = string.Empty;

                    // parse for vertexes

                    // add the frame
                    sf.Insert(Math.Min(index, sf.Count), tmpframe);
                }
            }

            this.animation = new AnimationSet<Key, FrameType>();
            this.animation.Frames.Add(sf.ToArray());
            // add named animations
            foreach (Key k in aniInfos.Keys)
            {
                this.animation.AddAnimation(k, aniInfos[k]);
            }

            return true;
        }

        //public bool Load(XmlNode animationNode)
        //{
        //    // create animation object
        //    Animation.AnimationSet<Animation.SpriteFrame> tmpAnimation = new Zider.Animation.AnimationSet<Zider.Animation.SpriteFrame>();

        //    // read in animations
        //    System.Xml.XmlNodeList anodes = animationNode;
        //    //anodes = aninode.SelectNodes(Resources.XML_animation);

        //    // process any animations that were found
        //    foreach (System.Xml.XmlNode an in anodes)
        //    {
        //        // read actor type\
        //        int startFrame = 0;
        //        int endFrame = 0;
        //        int frameRate = 30;
        //        bool looping = true;
        //        string name = string.Empty;

        //        // read animation name
        //        if (!Helpers.ParseStringNode(an, Resources.XML_name, out name))
        //        {
        //            this.err = Resources.MissingNamedElement.Replace(Resources.RKEY, Resources.XML_name);
        //            return false;
        //        }
        //        name = name.Trim();
        //        // make sure name is unuique
        //        if (tmpAnimation.animations.ContainsKey(name))
        //        {
        //            this.err = Resources.DuplicateAnimationNames;
        //            return false;
        //        }

        //        // read start frame
        //        if (!Helpers.ParseIntNode(an, Resources.XML_startframe, out startFrame))
        //        {
        //            this.err = Resources.MissingNamedElement.Replace(Resources.RKEY, Resources.XML_startframe);
        //            return false;
        //        }
        //        // read end frame
        //        if (!Helpers.ParseIntNode(an, Resources.XML_endframe, out endFrame))
        //        {
        //            this.err = Resources.MissingNamedElement.Replace(Resources.RKEY, Resources.XML_endframe);
        //            return false;
        //        }
        //        // read frame rate
        //        if (!Helpers.ParseIntNode(an, Resources.XML_framerate, out frameRate))
        //        {
        //            this.err = Resources.MissingNamedElement.Replace(Resources.RKEY, Resources.XML_framerate);
        //            return false;
        //        }
        //        // read looping
        //        if (!Helpers.ParseBoolNode(an, Resources.XML_loops, out looping))
        //        {
        //            this.err = Resources.MissingNamedElement.Replace(Resources.RKEY, Resources.XML_loops);
        //            return false;
        //        }

        //        // add the animation info
        //        tmpAnimation.AddAnimation(name, new Zider.Animation.AnimationInfo(startFrame, endFrame, frameRate, looping));

        //    }

        //    // read frames 
        //    System.Xml.XmlNodeList listItems = doc.DocumentElement.GetElementsByTagName(Resources.XML_frames);
        //    foreach (System.Xml.XmlNode framenode in listItems)
        //    {
        //        System.Xml.XmlNodeList fnodes;
        //        fnodes = framenode.SelectNodes(Resources.XML_frame);

        //        // process any frames that were found
        //        foreach (System.Xml.XmlNode fn in fnodes)
        //        {
        //            string err = string.Empty;
        //            // get frame index
        //            int frameIndex;
        //            Dictionary<string, string> attr;
        //            Helpers.ParseAttributes(fn, out attr);
        //            if (!attr.ContainsKey(Resources.XML_index))
        //            {
        //                this.err = Resources.MissingNamedElement.Replace(Resources.RKEY, Resources.XML_index);
        //                return false;
        //            }

        //            // try to get a number from value
        //            if (!int.TryParse(attr[Resources.XML_index], out frameIndex))
        //            {
        //                this.err = Resources.ExpectedNumber;
        //                return false;
        //            }

        //            // get texture
        //            string textureAsset;
        //            if (!Helpers.ParseStringNode(fn, Resources.XML_texture, out textureAsset))
        //            {
        //                this.err = Resources.MissingNamedElement.Replace(Resources.RKEY, Resources.XML_texture);
        //                return false;
        //            }

        //            // get origin 
        //            Vector2 origin = Vector2.Zero;
        //            attr.Clear();
        //            attr = null;
        //            if (!Helpers.ParseNodeAttributes(fn, Resources.XML_origin, out attr, out err))
        //            {
        //                this.err = err;
        //                return false;
        //            }
        //            // get origin data
        //            if (attr.ContainsKey(Resources.XML_x) && !float.TryParse(attr[Resources.XML_x], out origin.X))
        //            {
        //                this.err = Resources.InvalidPositionSpecified;
        //                return false;
        //            }
        //            if (attr.ContainsKey(Resources.XML_y) && !float.TryParse(attr[Resources.XML_y], out origin.Y))
        //            {
        //                this.err = Resources.InvalidPositionSpecified;
        //                return false;
        //            }

        //            // read color
        //            Color col = Color.White;
        //            if (!Helpers.ParseColorNode(fn, Resources.XML_color, out col, out err))
        //            {
        //                this.err = err;
        //                return false;
        //            }

        //            // read source rect
        //            Rectangle rect;
        //            if (!Helpers.ParseIntNode(fn, Resources.XML_sourceleft, out rect.X))
        //            {
        //                this.err = Resources.MissingNamedElement.Replace(Resources.RKEY, Resources.XML_sourceleft);
        //                return false;
        //            }
        //            if (!Helpers.ParseIntNode(fn, Resources.XML_sourcetop, out rect.Y))
        //            {
        //                this.err = Resources.MissingNamedElement.Replace(Resources.RKEY, Resources.XML_sourcetop);
        //                return false;
        //            }
        //            if (!Helpers.ParseIntNode(fn, Resources.XML_sourcewidth, out rect.Width))
        //            {
        //                this.err = Resources.MissingNamedElement.Replace(Resources.RKEY, Resources.XML_sourcewidth);
        //                return false;
        //            }
        //            if (!Helpers.ParseIntNode(fn, Resources.XML_sourceheight, out rect.Height))
        //            {
        //                this.err = Resources.MissingNamedElement.Replace(Resources.RKEY, Resources.XML_sourceheight);
        //                return false;
        //            }

        //            // TODO: load vertex data into sprite frame ...

        //            // create the frame
        //            Animation.SpriteFrame sFrame = new Zider.Animation.SpriteFrame();
        //            sFrame.color = col;
        //            sFrame.origin = origin;
        //            sFrame.sourceRect = rect;
        //            try
        //            {
        //                // try to load the texture
        //                sFrame.texture = this.game.Content.Load<Texture2D>(textureAsset);
        //                // add the frame
        //                tmpAnimation.Frames.Insert(frameIndex, sFrame);
        //            }
        //            catch (Exception)
        //            {
        //                this.err = Resources.CouldNotLoadTexture + Resources.TheValueWas.Replace(Resources.RKEY, textureAsset);
        //                return false;
        //            }
        //            finally
        //            {
        //                sFrame = null;
        //            }
        //        }
        //    }

        //    // set animation
        //    this.animation = tmpAnimation;
        //    // do cleanup
        //    tmpAnimation = null;

        //    doc = null;
        //    return true;
        //}

        /// <summary>
        /// Loads animation data from a file.
        /// </summary>
        /// <param name="filename">The path to the file that will be loaded.</param>
        /// <returns>Returns true if the data was loaded successfully.</returns>
        public bool Load(string filename)
        {
            // check if file exists
            if (!System.IO.File.Exists(filename))
            {
                this.errorMsg = Resources.FileDoesNotExist;
                return false;
            }

            // try to load file
            XmlDocument doc = new XmlDocument();
            try
            {
                doc.Load(filename);
            }
            catch
            {
                this.errorMsg = Resources.CouldNotLoadFileName.Replace(Resources.RKEY, System.IO.Path.GetFileName(filename));
                return false;
            }

            bool value = this.Load(doc);
            doc = null;
            return value;
        }

        /// <summary>
        /// LoaDs animation data from a stream.
        /// </summary>
        /// <param name="stream">A reference to a stream containing the xml data to load.</param>
        /// <returns>Returns true if the data was loaded successfully.</returns>
        public bool Load(System.IO.Stream stream)
        {
            // try to load file
            XmlDocument doc = new XmlDocument();
            try
            {
                doc.Load(stream);
            }
            catch
            {
                this.errorMsg = Resources.CouldNotLoadStream;
                return false;
            }

            bool value = this.Load(doc);
            doc = null;
            return value;
        }


        #region IDisposable Members

        /// <summary>
        /// Disposes of the object.
        /// </summary>
        /// <remarks>Sets internal animation varible to null.</remarks>
        public void Dispose()
        {
            this.animation = null;
        }

        #endregion
    }
}
