﻿/*
Code written/modified by: Dean Lunz (aka Crated by: X)
Web: http://www.createdbyx.com/ 
NOTICE: The code in this file does not fall under the zeeAni licence.
  
 Original source for this code comes from the tutorial "Animation Sets" found on the
 http://www.codeanxiety.com/ website.
 Original Code Author: Justin "Kukyona" O'Dell -> http://codeanxiety.com/user/Profile.aspx?UserID=2100
 */
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using zeeAni.Properties;
using xCollectiveXNA;


namespace zeeAni
{
    /// <summary>
    /// AnimationSet will help you animate your sprites or models.
    /// </summary>
    /// <typeparam name="FrameType">The type that will be used for the frames.</typeparam>
    /// <typeparam name="Key">A type that will be used as an indexer.</typeparam>
    public sealed class AnimationSet<Key, FrameType>
    {
        /// <summary>
        /// Holds a collection of animations.
        /// </summary>
        internal GenericDictionary<Key, AnimationInfo> animations = new GenericDictionary<Key, AnimationInfo>();
        /// <summary>
        /// Holds the current animation info.
        /// </summary>
        private AnimationInfo currentAnimationInfo = null;
        /// <summary>
        /// Holds the list of frames for all the animations.
        /// </summary>
        private GenericCollection<FrameType> frames = new GenericCollection<FrameType>();
        /// <summary>
        /// Holds the current frame.
        /// </summary>
        public float frame = 0f;
        private int lastFrameIndex = 0;
        private int currentFrameIndex = 0;
        public bool animating = false;
        public bool animationFinished = false;
        internal Key currentAnimationName;


        /// <summary>
        /// Removes all animations.
        /// </summary>
        public void Clear()
        {
            this.animations.Clear();
            this.currentAnimationInfo = null;
            this.currentAnimationName = default(Key);
        }

        /// <summary>
        /// Try to a animation using it's id.
        /// </summary>
        /// <param name="name">The id of the animation.</param>
        /// <param name="info">Will return a reference to a AnimationInfo object for the animation.</param>
        /// <returns>If true the animation info was successfully returned.</returns>
        public bool TryGetInfo(Key name, out AnimationInfo info)
        {
            return this.animations.TryGetValue(name, out info);
        }

        public Key[] AnimationNames
        {
            get
            {
                Key[] names = new Key[this.animations.Count];
                this.animations.Keys.CopyTo(names, 0);
                return names;
            }
        }

        /// <summary>
        /// Gets the current animation name.
        /// </summary>
        public Key CurrentAnimationName
        {
            get { return this.currentAnimationName; }
        }

        /// <summary>
        /// Default constructor.
        /// </summary>
        public AnimationSet()
        {
        }

        /// <summary>
        /// Default constructor.
        /// </summary>
        /// <param name="frames">Frame data that we will be working with.</param>
        public AnimationSet(GenericCollection<FrameType> frames)
        {
            this.frames = frames;

            // add a default animation that loops and encompasses all frames
            //AddAnimation(Resources.Default, new SpriteAnimationInfo(0, frames.Length - 1, 15f, true));
            //currentAnimationInfo = animations[Resources.Default];
        }

        /// <summary>
        /// Adds a new animation sing the provided info.
        /// </summary>
        /// <param name="name">The name of the animation.</param>
        /// <param name="info">A reference to an AnimationInfo object.</param>
        /// <returns>Returns true if the animation was added. If false it means that a animation with the same name has already been added or the info parameter is was null.</returns>
        public bool AddAnimation(Key name, AnimationInfo info)
        {
            if (info == null) return false;

            // if we already have an animation with this name then ignore the new one
            if (animations.ContainsKey(name))
                return false;

            animations.Add(name, info);
            return true;
        }

        /// <summary>
        /// Resets the current animation, if playing, and starts animating.
        /// </summary>
        public void Play()
        {
            // hitting play starts the animation from frame 0
            currentFrameIndex = currentAnimationInfo.StartFrame;
            frame = 0f;
            animationFinished = false;

            // only animate if we actually need to
            if (currentAnimationInfo.StartFrame != currentAnimationInfo.EndFrame)
                animating = true;
        }

        /// <summary>
        /// Starts playing the animation with the specified name.
        /// </summary>
        /// <param name="name">The name of the animation to play.</param>
        public void Play(Key name)
        {
#if DEBUG
            System.Diagnostics.Debug.WriteLine("Playing animation: " + name.ToString());
#endif
            // do we even have an animation with this name?
            if (animations.ContainsKey(name) != true)
                return;

            // set the current animation
            currentAnimationInfo = animations[name];
            this.currentAnimationName = name;

            Play();
        }

        /// <summary>
        /// Starts playing the animation with the specified name.
        /// </summary>
        /// <param name="name">The name of the animation to play.</param>
        /// <param name="offset">The value that will be passed to the CurrentFrameOffset property.</param>
        public void Play(Key name, float offset)
        {
            this.Play(name);
            this.CurrentFrameOffset = offset;
        }

        /// <summary>
        /// Pauses the animation at its current frame.
        /// </summary>
        public void Pause()
        {
            animating = false;
        }

        /// <summary>
        /// Resumes playing the current animation.
        /// </summary>
        public void Resume()
        {
            // only animate if we actually need to, single frame animations don't need to be animated at all
            if (currentAnimationInfo.StartFrame != currentAnimationInfo.EndFrame)
                animating = true;
        }

        /// <summary>
        /// Stop playing the current animation and resets it to the first frame.
        /// </summary>
        public void Stop()
        {
            // stopping resets the movie back to frame 0
            currentFrameIndex = currentAnimationInfo.StartFrame;
            frame = 0f;
            animating = false;
            animationFinished = false;
        }

        /// <summary>
        /// Called once per frame to help animate.
        /// </summary>
        /// <param name="gameTime">Snapshot of the current game timing.</param>
        public void Update(GameTime gameTime)
        {
            // if we aren't animating then theres nothing to do
            if (animating != true || currentAnimationInfo == null)
                return;

            // increment the current frame position
            frame += currentAnimationInfo.FrameRate * (float)gameTime.ElapsedGameTime.TotalSeconds;

            int totalFrames = currentAnimationInfo.EndFrame - currentAnimationInfo.StartFrame + 1;

            // have we hit our limit?
            if ((int)frame >= totalFrames)
            {
                // if the animation doesn't loop then just pause at the last frame
                if (currentAnimationInfo.Loops != true)
                {
                    frame = (float)currentAnimationInfo.EndFrame;
                    animating = false;
                    animationFinished = true;
                }
                // otherwise, just move backwards to frame 0
                else
                    // we subtract here that way if we were at frame 2.02 we'll start at frame 0.02
                    frame -= (float)totalFrames;
            }

            // have we changed frames?
            if ((int)frame != currentFrameIndex)
            {
                lastFrameIndex = currentFrameIndex;
                currentFrameIndex = currentAnimationInfo.StartFrame + (int)frame;
            }
        }

        /// <summary>
        /// Gets the array of frames in this animation set.
        /// </summary>
        public GenericCollection<FrameType> Frames
        {
            get { return frames; }
        }

        /// <summary>
        /// Gets the current animation being played.
        /// </summary>
        public AnimationInfo CurrentAnimationInfo
        {
            get { return currentAnimationInfo; }
        }

        /// <summary>
        /// Gets whether or not the current animation has finished playing.
        /// </summary>
        public bool AnimationFinished
        {
            get { return animationFinished; }
        }

        /// <summary>
        /// Gets the current frame being played.
        /// </summary>
        public FrameType CurrentFrame
        {
            get { return frames[currentFrameIndex]; }
        }

        /// <summary>
        /// Gets the current frame index being played.
        /// </summary>
        public int CurrentFrameIndex
        {
            get { return currentFrameIndex; }
        }

        /// <summary>
        /// Gets/Sets the current frame offset.
        /// </summary>
        /// <remarks>
        /// This property will get or set the current frame offset relative to the start frame.
        /// This is usefull when you have more then one animation of the same length and want to play a different animation but want to preserve
        /// the current frame offset. See example.
        /// </remarks>
        /// <example>
        /// StartFrame = 8<br />
        /// EndFrame = 10<br />
        /// If CurrentFrameOffset is set to 1 the CurrentFrameIndex property would equal 9. <br /><br />
        /// <code>
        ///    zeeAni.AnimationInfo info = this.animation.CurrentAnimationInfo; <br />
        ///    float oldIndex = 0;<br />
        ///    if (info != null) oldIndex = this.animation.CurrentFrameOffset;<br />
        ///    this.animation.Play(Resources.ANI_Walking + " " + this.spriteDir.ToString());<br />
        ///    if (info != null) this.animation.CurrentFrameOffset = oldIndex;
        ///</code>
        ///</example>
        public float CurrentFrameOffset
        {
            get { return this.frame; }
            set
            {
                if (this.currentAnimationInfo == null) return;
                this.frame = MathHelper.Clamp(value, 0, this.currentAnimationInfo.EndFrame - this.currentAnimationInfo.StartFrame);
            }
        }

        /// <summary>
        /// Gets the last frame that was played.
        /// </summary>
        public FrameType LastFrame
        {
            get { return frames[lastFrameIndex]; }
        }

        /// <summary>
        /// Gets the index of the last frame that was played.
        /// </summary>
        public int LastFrameIndex
        {
            get { return lastFrameIndex; }
        }

        /// <summary>
        /// Gets the ratio of how far into the current frame we are in.
        /// 0.0 = 0%, 1.0 = 100%
        /// </summary>
        public float BlendRatio
        {
            get { return frame - (float)((int)frame); }
        }
    }
}
