/*
Created by: X
http://www.createdbyx.com/

The Zider engine is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System.Reflection;
using zeeAni.Properties;


namespace zeeAni
{
    /// <summary>
    /// Contains various helper methods.
    /// </summary>
    public static class Helpers
    {
        /// <summary>
        /// Used to determin is a value is numeric.
        /// </summary>
        /// <param name="value">The value to check is it is numberic.</param>
        /// <returns>Will return true is the value is a numberic value.</returns>
        public static bool IsNumeric(string value)
        {
            if (value == null) return false;
            value = value.Trim();
            if (value == string.Empty) return false;
            if (char.IsSymbol(value[0]) == false && char.IsNumber(value[0]) == false) return false;
            if (char.IsSymbol(value[0]) && value.Length > 1 && char.IsNumber(value[1]) == false) return false;
            return true;
        }

        /// <summary>
        /// Parses the attributes from a node and returns there values.
        /// </summary>
        /// <param name="node">A reference to a node to get the attributes from.</param>
        /// <param name="list">The list that will be returned containing the discovered values.</param>
        public static void ParseAttributes(System.Xml.XmlNode node, out Dictionary<string, string> list)
        {

            Dictionary<string, string> items = new Dictionary<string, string>();
            for (int idx = 0; idx <= node.Attributes.Count - 1; idx++)
            {
                items.Add(node.Attributes[idx].Name, node.Attributes[idx].InnerText);
            }
            list = items;
        }

        /// <summary>
        /// Parses color information from a node.
        /// </summary>
        /// <param name="node">A reference to a node to get the color data from.</param>
        /// <param name="c">A color value that will be returned.</param>
        /// <returns>Returns true if the color was parsed.</returns>
        /// <example>
        /// The expected format is as fallows...<br/>
        /// &lt;color r="0" g="0" b="0" a="0" /&gt;
        /// </example>
        private static bool ParseColor(System.Xml.XmlNode node, out Color c)
        {
            c = Color.White;
            System.Xml.XmlNode rn, gn, bn, an;
            rn = node.Attributes.GetNamedItem(Resources.XML_r);
            gn = node.Attributes.GetNamedItem(Resources.XML_g);
            bn = node.Attributes.GetNamedItem(Resources.XML_b);
            an = node.Attributes.GetNamedItem(Resources.XML_a);

            byte rv = 255;
            byte gv = 255;
            byte bv = 255;
            byte av = 255; // set default alpha value

            // extract color components
            if (rn != null && !byte.TryParse(rn.InnerText, out rv))
            {
                return false;
            }
            if (gn != null && !byte.TryParse(gn.InnerText, out gv))
            {
                return false;
            }
            if (bn != null && !byte.TryParse(bn.InnerText, out bv))
            {
                return false;
            }
            if (an != null && !byte.TryParse(an.InnerText, out av))
            {
                // we can ignore alpha
            }

            // assign color
            c = new Color(rv, gv, bv, av);
            return true;
        }

        /// <summary>
        /// Finds a node and returns it's attributes.
        /// </summary>
        /// <param name="parentnode">The node that contains the node to search for.</param>
        /// <param name="name">The name of the node to search for.</param>
        /// <param name="value">Will contain the attributes of the node is found.</param>
        /// <returns>Returns true if the node was found.</returns>
        public static bool ParseNodeAttributes(System.Xml.XmlNode parentnode, string name, out Dictionary<string, string> value)
        {
            string err;
            return ParseNodeAttributes(parentnode, name, out value, out err);
        }

        /// <summary>
        /// Finds a node and returns it's attributes.
        /// </summary>
        /// <param name="parentnode">The node that contains the node to search for.</param>
        /// <param name="name">The name of the node to search for.</param>
        /// <param name="value">Will contain the attributes of the node is found.</param>
        /// <param name="err">Will contain an error message if an error occoured.</param>
        /// <returns>Returns true if the node was found.</returns>
        public static bool ParseNodeAttributes(System.Xml.XmlNode parentnode, string name, out Dictionary<string, string> value, out string err)
        {
            value = null;
            System.Xml.XmlNode node = parentnode.SelectSingleNode(name);
            if (node == null)
            {
                err = Resources.CouldNotFindNamedElement.Replace(Resources.RKEY, name);
                return false;
            }
            Helpers.ParseAttributes(node, out value);
            err = string.Empty;
            return true;
        }

        /// <summary>
        /// Parses a parent node ans searches for a specified child node.
        /// </summary>
        /// <param name="parentnode">The parent node to search under.</param>
        /// <param name="name">The name of the node to search for.</param>
        /// <param name="value">Will contain the value of the node if the node was found.</param>
        /// <returns>Returns true if succsessfull.</returns>
        public static bool ParseStringNode(System.Xml.XmlNode parentnode, string name, out string value)
        {
            string err;
            return ParseStringNode(parentnode, name, out value, out err);
        }

        /// <summary>
        /// Parses a parent node ans searches for a specified child node.
        /// </summary>
        /// <param name="parentnode">The parent node to search under.</param>
        /// <param name="name">The name of the node to search for.</param>
        /// <param name="value">Will contain the value of the node if the node was found.</param>
        /// <param name="err">Will contain an error message if an error occoured.</param>
        /// <returns>Returns true if succsessfull.</returns>
        public static bool ParseStringNode(System.Xml.XmlNode parentnode, string name, out string value, out string err)
        {
            value = string.Empty;
            System.Xml.XmlNode node = parentnode.SelectSingleNode(name);
            if (node == null)
            {
                err = Resources.CouldNotFindNamedElement.Replace(Resources.RKEY, name);
                return false;
            }
            value = node.InnerText;
            err = string.Empty;
            return true;
        }

        /// <summary>
        /// Parses a parent node for a specified child node and returns it's value as an integer.
        /// </summary>
        /// <param name="parentnode">The parent node to search under.</param>
        /// <param name="name">The name of the node to search for.</param>
        /// <param name="value">Will contain the value of the node if the node was found.</param>
        /// <param name="err">Will contain an error message if an error occoured.</param>
        /// <returns>Returns true if succsessfull.</returns>
        public static bool ParseIntNode(System.Xml.XmlNode parentnode, string name, ref int value)
        {
            string err;
            return ParseIntNode(parentnode, name, ref value, out err);
        }

        /// <summary>
        /// Parses a parent node for a specified child node and returns it's value as an integer.
        /// </summary>
        /// <param name="parentnode">The parent node to search under.</param>
        /// <param name="name">The name of the node to search for.</param>
        /// <param name="value">Will contain the value of the node if the node was found.</param>
        /// <param name="err">Will contain an error message if an error occoured.</param>
        /// <returns>Returns true if succsessfull.</returns>
        public static bool ParseIntNode(System.Xml.XmlNode parentnode, string name, ref int value, out string err)
        {
            int newValue = new int();
            System.Xml.XmlNode node = parentnode.SelectSingleNode(name);
            if (node == null)
            {
                err = Resources.CouldNotFindNamedElement.Replace(Resources.RKEY, name);
                return false;
            }
            if (!int.TryParse(node.InnerText, out newValue))
            {
                err = Resources.ValueIsNotInteger + " " + Resources.NamedElement.Replace(Resources.RKEY, name);
                return false;
            }
            err = string.Empty;
            value = newValue;
            return true;
        }

        /// <summary>
        /// Parses color information from a xml node.
        /// </summary>
        /// <param name="parentnode">The parent node to search under.</param>
        /// <param name="value">A color value that will be returned.</param>
        /// <param name="err">A error string that will be returned is an error occoured.</param>
        /// <returns>Returns true if the color was parsed.</returns>
        /// <example>
        /// The expected format is as fallows...<br/>
        /// &lt;color r="0" g="0" b="0" a="0" /&gt;
        /// </example>
        public static bool ParseColorNode(System.Xml.XmlNode parentnode, string name, out Color value, out string err)
        {
            Dictionary<string, string> items;
            if (!Helpers.ParseNodeAttributes(parentnode, name, out items, out err))
            {
                value = Color.White;
                err = Resources.ExpectedNumber + " " + Resources.NamedElement.Replace(Resources.RKEY, name);
                return false;
            }

            byte rv, gv, bv, av;
            rv = 255;
            gv = 255;
            bv = 255;
            av = 255; // default alpha value

            // ensure that aleast the rgb components were specified
            if (!items.ContainsKey(Resources.XML_r) || !items.ContainsKey(Resources.XML_g) || !items.ContainsKey(Resources.XML_b))
            {
                value = Color.White;
                err = Resources.ExpectedNumber + " " + Resources.NamedElement.Replace(Resources.RKEY, name);
                return false;
            }

            // get rbg values
            if (!byte.TryParse(items[Resources.XML_r], out rv))
            {
                value = Color.White;
                err = Resources.ExpectedNumber + " " + Resources.NamedElement.Replace(Resources.RKEY, name);
                return false;
            }
            if (!byte.TryParse(items[Resources.XML_g], out gv))
            {
                value = Color.White;
                err = Resources.ExpectedNumber + " " + Resources.NamedElement.Replace(Resources.RKEY, name);
                return false;
            }
            if (!byte.TryParse(items[Resources.XML_b], out bv))
            {
                value = Color.White;
                err = Resources.ExpectedNumber + " " + Resources.NamedElement.Replace(Resources.RKEY, name);
                return false;
            }

            // get alpha value
            if (items.ContainsKey(Resources.XML_a) && !byte.TryParse(items[Resources.XML_a], out av))
            {
                value = Color.White;
                err = Resources.ExpectedNumber + " " + Resources.NamedElement.Replace(Resources.RKEY, name);
                return false;
            }

            // create color
            value = new Color(rv, gv, bv, av);

            return true;
        }

        /// <summary>
        /// Parses a parent node for a specified child node and returns it's value as an float.
        /// </summary>
        /// <param name="parentnode">The parent node to search under.</param>
        /// <param name="name">The name of the node to search for.</param>
        /// <param name="value">Will contain the value of the node if the node was found.</param>
        /// <returns>Returns true if succsessfull.</returns>
        public static bool ParseFloatNode(System.Xml.XmlNode parentnode, string name, ref float value)
        {
            string err;
            return ParseFloatNode(parentnode, name, ref value, out err);
        }

        /// <summary>
        /// Parses a parent node for a specified child node and returns it's value as an float.
        /// </summary>
        /// <param name="parentnode">The parent node to search under.</param>
        /// <param name="name">The name of the node to search for.</param>
        /// <param name="value">Will contain the value of the node if the node was found.</param>
        /// <param name="err">Will contain an error message if an error occoured.</param>
        /// <returns>Returns true if succsessfull.</returns>
        public static bool ParseFloatNode(System.Xml.XmlNode parentnode, string name, ref float value, out string err)
        {
            float newValue = new float();
           // float oldValue = value;             
            System.Xml.XmlNode node = parentnode.SelectSingleNode(name);
            if (node == null)
            {
                err = Resources.CouldNotFindNamedElement.Replace(Resources.RKEY, name);
              //  value = oldValue;
                return false;
            }
            if (!float.TryParse(node.InnerText, out newValue))
            {
                err = Resources.ValueIsNotFloat + " " + Resources.NamedElement.Replace(Resources.RKEY, name);
              //  value = oldValue;
                return false;
            }
            err = string.Empty;
            value = newValue;
            return true;
        }

        /// <summary>
        /// Parses a parent node for a specified child node and returns it's value as an boolean.
        /// </summary>
        /// <param name="parentnode">The parent node to search under.</param>
        /// <param name="name">The name of the node to search for.</param>
        /// <param name="value">Will contain the value of the node if the node was found.</param>
        /// <returns>Returns true if succsessfull.</returns>
        public static bool ParseBoolNode(System.Xml.XmlNode parentnode, string name, out bool value)
        {
            string err;
            return ParseBoolNode(parentnode, name, out value, out err);
        }

        /// <summary>
        /// Parses a parent node for a specified child node and returns it's value as an boolean.
        /// </summary>
        /// <param name="parentnode">The parent node to search under.</param>
        /// <param name="name">The name of the node to search for.</param>
        /// <param name="value">Will contain the value of the node if the node was found.</param>
        /// <param name="err">Will contain an error message if an error occoured.</param>
        /// <returns>Returns true if succsessfull.</returns>
        public static bool ParseBoolNode(System.Xml.XmlNode parentnode, string name, out bool value, out string err)
        {
            value = new bool();
            System.Xml.XmlNode node = parentnode.SelectSingleNode(name);
            if (node == null)
            {
                err = Resources.CouldNotFindNamedElement.Replace(Resources.RKEY, name);
                return false;
            }
            if (!bool.TryParse(node.InnerText, out value))
            {
                err = Resources.ValueIsNotBool + " " + Resources.NamedElement.Replace(Resources.RKEY, name);
                return false;
            }
            err = string.Empty;
            return true;
        }
    }
}
