/*
Created by: X
http://www.createdbyx.com/

The zeeAni library is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;


namespace zeeAni
{
    /// <summary>
    /// A object representing a frame of animation.
    /// </summary>
    public class SpriteFrame : ICloneable
    {
        /// <summary>
        /// Holds the origin or the frame.
        /// </summary>
        internal protected Vector2 origin;
        /// <summary>
        /// Holds a reference to a texture for the frame.
        /// </summary>
        internal protected Texture2D texture;
        /// <summary>
        /// Holds a color used when drawing the frame.
        /// </summary>
        internal protected Color color = Color.White;
        /// <summary>
        /// Holds the source rectangle information when drawing the frame.
        /// </summary>
        internal protected Rectangle sourceRect;
        /// <summary>
        /// Holds the effect info when drawing the frame.
        /// </summary>
        /// <remarks>The default is None.</remarks>
        protected internal SpriteEffects effect = SpriteEffects.None;
        /// <summary>
        /// Holds the rotation whe the frame is drawn.
        /// </summary>
        protected internal float rotation = 0;
        /// <summary>
        /// Holds the scale to use when drawing the frame.
        /// </summary>
        protected internal Vector2 scale = Vector2.One;


        /// <summary>
        /// Draws the sprite frame.
        /// </summary>
        /// <param name="sb">A reference to a SpriteBatch object.</param>
        /// <param name="X">The x position on the screen to draw.</param>
        /// <param name="Y">The y position on the screen to draw.</param>
        public void Draw(SpriteBatch sb, float X, float Y)
        {
            this.Draw(sb, X, Y, 0);
        }

        /// <summary>
        /// Draws the sprite frame.
        /// </summary>
        /// <param name="sb">A reference to a SpriteBatch object.</param>
        /// <param name="position">The position on the screen to draw.</param>
        public void Draw(SpriteBatch sb, Vector2 position)
        {
            this.Draw(sb, position.X, position.Y, 0);
        }

        /// <summary>
        /// Draws the sprite frame.
        /// </summary>
        /// <param name="sb">A reference to a SpriteBatch object.</param>
        /// <param name="X">The x position on the screen to draw.</param>
        /// <param name="Y">The y position on the screen to draw.</param>
        /// <param name="depth">The sorting depth of the sprite, between 0 (front) and 1 (back).</param>
        public void Draw(SpriteBatch sb, float X, float Y, float depth)
        {
            sb.Draw(this.texture, new Vector2(X, Y), this.sourceRect, this.color, this.rotation, this.origin, this.scale, this.effect, depth);
        }

        /// <summary>
        /// Draws the sprite frame.
        /// </summary>
        /// <param name="sb">A reference to a SpriteBatch object.</param>
        /// <param name="position">The position on the screen to draw.</param>
        /// <param name="depth">The sorting depth of the sprite, between 0 (front) and 1 (back).</param>
        public void Draw(SpriteBatch sb, Vector2 position, float depth)
        {
            this.Draw(sb, position.X, position.Y, depth);
        }
        
        /// <summary>
        /// Draws the sprite frame.
        /// </summary>
        /// <param name="sb">A reference to a SpriteBatch object.</param>
        /// <param name="X">The x position on the screen to draw.</param>
        /// <param name="Y">The y position on the screen to draw.</param>
        /// <param name="rotation">The angle, in radians, to rotate the sprite around the origin.</param>
        /// <param name="depth">The sorting depth of the sprite, between 0 (front) and 1 (back).</param>
        public void Draw(SpriteBatch sb, float X, float Y, float rotation, float depth)
        {
            sb.Draw(this.texture, new Vector2(X, Y), this.sourceRect, this.color, rotation, this.origin, this.scale, this.effect, depth);
        }

        /// <summary>
        /// Draws the sprite frame.
        /// </summary>
        /// <param name="sb">A reference to a SpriteBatch object.</param>
        /// <param name="position">The position on the screen to draw.</param>
        /// <param name="rotation">The angle, in radians, to rotate the sprite around the origin.</param>
        /// <param name="depth">The sorting depth of the sprite, between 0 (front) and 1 (back).</param>
        public void Draw(SpriteBatch sb, Vector2 position, float rotation, float depth)
        {
            this.Draw(sb, position.X, position.Y, rotation, depth);
        }

        /// <summary>
        /// Gets/Sets the scale to use when drawing the frame.
        /// </summary>
        public virtual Vector2 Scale
        {
            get { return scale; }
            set { scale = value; }
        }

        /// <summary>
        /// Gets/Sets the rotation whe the frame is drawn.
        /// </summary>
        public virtual float Rotation
        {
            get { return rotation; }
            set { rotation = value; }
        }

        /// <summary>
        /// Gets/Sets the effect info when drawing the frame.
        /// </summary>
        public virtual SpriteEffects Effect
        {
            get { return effect; }
            set { effect = value; }
        }

        /// <summary>
        /// Gets/Sets the source rectangle information when drawing the frame.
        /// </summary>
        public virtual Rectangle SourceRectangle
        {
            get { return sourceRect; }
            set { sourceRect = value; }
        }

        /// <summary>
        /// Gets/Sets the source rectangle left value when drawing the frame.
        /// </summary>
        public virtual int SourceLeft
        {
            get { return sourceRect.X; }
            set { sourceRect.X = value; }
        }

        /// <summary>
        /// Gets/Sets the source rectangle top value when drawing the frame.
        /// </summary>
        public virtual int SourceTop
        {
            get { return sourceRect.Y; }
            set { sourceRect.Y = value; }
        }

        /// <summary>
        /// Gets/Sets the source rectangle width value when drawing the frame.
        /// </summary>
        public virtual int SourceWidth
        {
            get { return sourceRect.Width; }
            set { sourceRect.Width = value; }
        }

        /// <summary>
        /// Gets/Sets the source rectangle height value when drawing the frame.
        /// </summary>
        public virtual int SourceHeight
        {
            get { return sourceRect.Height; }
            set { sourceRect.Height = value; }
        }

        /// <summary>
        /// Gets/Sets a color used when drawing the frame.
        /// </summary>
        public virtual Color Color
        {
            get { return color; }
            set { color = value; }
        }

        /// <summary>
        /// Gets/Sets a reference to a texture for the frame.
        /// </summary>
        public virtual Texture2D Texture
        {
            get { return texture; }
            set { texture = value; }
        }

        /// <summary>
        /// Gets/Sets the origin or the frame.
        /// </summary>
        public virtual Vector2 Origin
        {
            get { return origin; }
            set { origin = value; }
        }

        /// <summary>
        /// Initilizes a new instance of the SpriteFrame object.
        /// </summary>
        public SpriteFrame()
        {
        }

        /// <summary>
        /// Initilizes a new instance of the SpriteFrame object.
        /// </summary>
        /// <param name="tex">A reference to a texture to use whn drawing the frame.</param>
        /// <param name="origin">A origin to use when drawing the frame.</param>
        public SpriteFrame(Texture2D tex, Vector2 origin)
        {
            this.texture = tex;
            this.sourceRect = new Rectangle(0, 0, this.texture.Width, this.texture.Height);
            this.origin = origin;
        }

        /// <summary>
        /// Initilizes a new instance of the SpriteFrame object.
        /// </summary>
        /// <param name="tex">A reference to a texture to use whn drawing the frame.</param>
        public SpriteFrame(Texture2D tex)
        {
            this.texture = tex;
            this.sourceRect = new Rectangle(0, 0, this.texture.Width, this.texture.Height);
        }

        #region ICloneable Members

        /// <summary>
        /// Clones the object.
        /// </summary>
        /// <returns>Returns a copy of the object.</returns>
        public virtual object Clone()
        {
            SpriteFrame frame = new SpriteFrame();
            frame.color = this.color;
            frame.effect = this.effect;
            frame.origin = this.origin;
            frame.rotation = this.rotation;
            frame.scale = this.scale;
            frame.sourceRect = this.sourceRect;
            frame.texture = this.texture;
            return frame;
        }

        #endregion
    }
}
